@@ -16,13 +16,17 @@
 from __future__ import annotations
 
 import logging
+import warnings
 from functools import partial
-from typing import List, Tuple
+from typing import Iterable, List, Tuple
 
 import torch
 import torch.nn as nn
-from opacus.layers.dp_rnn import DPRNNBase, DPRNNCellBase, RNNLinear
+from opacus.grad_sample.functorch import ft_compute_per_sample_gradient, prepare_layer
+from opacus.grad_sample.gsm_base import AbstractGradSampleModule
+from opacus.layers.dp_rnn import DPGRU, DPLSTM, DPRNN, RNNLinear
 from opacus.utils.module_utils import (
+    has_trainable_params,
     requires_grad,
     trainable_modules,
     trainable_parameters,
@@ -31,8 +35,6 @@ from opacus.utils.module_utils import (
 
 logger = logging.getLogger(__name__)
 
-OPACUS_PARAM_MONKEYPATCH_ATTRS = ["_forward_counter", "_current_grad_sample"]
-
 
 def create_or_accumulate_grad_sample(
     *, param: torch.Tensor, grad_sample: torch.Tensor, max_batch_len: int
@@ -73,10 +75,14 @@ def promote_current_grad_sample(p: nn.Parameter) -> None:
         del p._current_grad_sample
 
 
-class GradSampleModule(nn.Module):
-    r"""
-    Extends nn.Module so that its parameter tensors have an extra field called .grad_sample.
+class GradSampleModule(AbstractGradSampleModule):
     """
+    Hooks-based implementation of AbstractGradSampleModule
+
+    Computes per-sample gradients using custom-written methods for each layer.
+    See README.md for more details
+    """
+
     GRAD_SAMPLERS = {}
 
     def __init__(
@@ -86,6 +92,7 @@ class GradSampleModule(nn.Module):
         batch_first=True,
         loss_reduction="mean",
         strict: bool = True,
+        force_functorch=False,
     ):
         """
 
@@ -105,13 +112,20 @@ class GradSampleModule(nn.Module):
                 possible and set to None otherwise. This is not recommended, because
                 some unsupported modules (e.g. BatchNorm) affect other parameters and
                 invalidate the concept of per sample gradients for the entire model.
+            force_functorch: If set to ``True``, will use functorch to compute
+                all per sample gradients. Otherwise, functorch will be used only
+                for layers without registered grad sampler methods.
 
         Raises:
             NotImplementedError
                 If ``strict`` is set to ``True`` and module ``m`` (or any of its
                 submodules) doesn't have a registered grad sampler function.
         """
-        super().__init__()
+        super().__init__(
+            m,
+            batch_first=batch_first,
+            loss_reduction=loss_reduction,
+        )
 
         errors = self.validate(module=m, strict=strict)
         if errors and not strict:
@@ -120,81 +134,40 @@ class GradSampleModule(nn.Module):
                 "Using non-strict mode, continuing"
             )
 
-        self._module = m
         self.hooks_enabled = False
         self.batch_first = batch_first
         self.loss_reduction = loss_reduction
-        self.add_hooks(loss_reduction=loss_reduction, batch_first=batch_first)
-
-        for _, p in trainable_parameters(self):
-            p.grad_sample = None
-            p._forward_counter = 0
-
-    def __getattr__(self, item):
-        try:
-            return super().__getattr__(item)
-        except AttributeError as e:
-            submodules = dict(self._module.named_modules())
-            if item and item in submodules:
-                return submodules[item]
-            raise e
+        self.force_functorch = force_functorch
+        self.add_hooks(
+            loss_reduction=loss_reduction,
+            batch_first=batch_first,
+            force_functorch=force_functorch,
+        )
 
     def forward(self, *args, **kwargs):
         return self._module(*args, **kwargs)
 
-    def zero_grad(self, set_to_none: bool = False):
-        """
-        Clear gradients.
+    def iterate_submodules(self, module: nn.Module) -> Iterable[nn.Module]:
+        if has_trainable_params(module):
+            yield module
 
-        Clears ``p.grad`` and ``p.grad_sample`` for all of it's parameters
+        # Don't recurse if module is handled by functorch
+        if (
+            has_trainable_params(module)
+            and type(module) not in self.GRAD_SAMPLERS
+            and type(module) not in [DPRNN, DPLSTM, DPGRU]
+        ):
+            return
 
-        Notes:
-            ``set_to_none`` argument only affects ``p.grad``. ``p.grad_sample`` is
-            never zeroed out and always set to None.
-            Normal grads can do this, because their shape is always the same.
-            Grad samples do not behave like this, as we accumulate gradients from different
-            batches in a list
-
-        Args:
-            set_to_none: instead of setting to zero, set the grads to None. (only
-            affects regular gradients. Per sample gradients are always set to None)
-        """
-        if set_to_none is False:
-            logger.info(
-                "Despite set_to_none is set to False, "
-                "opacus will set p.grad_sample to None due to "
-                "non-trivial gradient accumulation behaviour"
-            )
-        self.set_grad_sample_to_none()
-        super().zero_grad(set_to_none)
-
-    def set_grad_sample_to_none(self):
-        """
-        Sets ``.grad_sample`` to None
-        """
-        for _, p in trainable_parameters(self):
-            p.grad_sample = None
-
-    def del_grad_sample(self):
-        """
-        Deleted ``.grad_sample`` attribute from all model parameters
-        """
-        for _, p in trainable_parameters(self):
-            del p.grad_sample
-
-    def to_standard_module(self) -> nn.Module:
-        """
-        Returns the standard nn.Module wrapped by this, eliminating all traces
-        of grad samples and hooks
-
-        Returns:
-            The wrapped module
-        """
-        self._close()
-        return self._module
+        for m in module.children():
+            yield from self.iterate_submodules(m)
 
     def add_hooks(
-        self, *, loss_reduction: str = "mean", batch_first: bool = True
+        self,
+        *,
+        loss_reduction: str = "mean",
+        batch_first: bool = True,
+        force_functorch: bool = False,
     ) -> None:
         """
         Adds hooks to model to save activations and backprop values.
@@ -211,6 +184,8 @@ class GradSampleModule(nn.Module):
                 ``[K, batch_size, ...]``
             loss_reduction: Indicates if the loss reduction (for aggregating the gradients)
                 is a sum or a mean operation. Can take values "sum" or "mean"
+            force_functorch: If set to ``True``, will use functorch to compute all per sample gradients.
+                Otherwise, functorch will be used only for layers without registered grad sampler methods.
         """
         if hasattr(self._module, "autograd_grad_sample_hooks"):
             raise ValueError("Trying to add hooks twice to the same model")
@@ -218,21 +193,28 @@ class GradSampleModule(nn.Module):
             self._module.autograd_grad_sample_hooks = []
             self.autograd_grad_sample_hooks = self._module.autograd_grad_sample_hooks
 
-        for _module_name, module in trainable_modules(self._module):
-            if type(module) in self.GRAD_SAMPLERS:
-                self.autograd_grad_sample_hooks.append(
-                    module.register_forward_hook(self.capture_activations_hook)
-                )
+        for module in self.iterate_submodules(self._module):
+            # Do not add hooks to DPRNN, DPLSTM or DPGRU as the hooks are handled by the `RNNLinear`
+            if type(module) in [DPRNN, DPLSTM, DPGRU]:
+                continue
 
-                self.autograd_grad_sample_hooks.append(
-                    module.register_backward_hook(
-                        partial(
-                            self.capture_backprops_hook,
-                            loss_reduction=loss_reduction,
-                            batch_first=batch_first,
-                        )
+            if force_functorch or not type(module) in self.GRAD_SAMPLERS:
+                prepare_layer(module, batch_first=batch_first)
+
+            self.autograd_grad_sample_hooks.append(
+                module.register_forward_hook(self.capture_activations_hook)
+            )
+
+            self.autograd_grad_sample_hooks.append(
+                module.register_backward_hook(
+                    partial(
+                        self.capture_backprops_hook,
+                        loss_reduction=loss_reduction,
+                        batch_first=batch_first,
                     )
                 )
+            )
+
         self.enable_hooks()
 
     def remove_hooks(self) -> None:
@@ -257,6 +239,11 @@ class GradSampleModule(nn.Module):
             delattr(self, "autograd_grad_sample_hooks")
             delattr(self._module, "autograd_grad_sample_hooks")
 
+        # Remove functorch hooks
+        for _module_name, module in trainable_modules(self._module):
+            if hasattr(module, "ft_compute_sample_grad"):
+                delattr(module, "ft_compute_sample_grad")
+
     def disable_hooks(self) -> None:
         r"""
         Globally disable all hooks installed by this library.
@@ -273,19 +260,9 @@ class GradSampleModule(nn.Module):
         """
         self.hooks_enabled = True
 
-    def __repr__(self):
-        return f"GradSampleModule({self._module.__repr__()})"
-
     def _close(self):
-        self.del_grad_sample()
+        super()._close()
         self.remove_hooks()
-        self._clean_up_attributes()
-
-    def _clean_up_attributes(self):
-        for attr in OPACUS_PARAM_MONKEYPATCH_ATTRS:
-            for p in self.parameters():
-                if hasattr(p, attr):
-                    delattr(p, attr)
 
     def capture_activations_hook(
         self,
@@ -305,7 +282,7 @@ class GradSampleModule(nn.Module):
 
         if not hasattr(module, "activations"):
             module.activations = []
-        module.activations.append(forward_input[0].detach())  # pyre-ignore
+        module.activations.append([t.detach() for t in forward_input])  # pyre-ignore
 
         for _, p in trainable_parameters(module):
             p._forward_counter += 1
@@ -352,7 +329,11 @@ class GradSampleModule(nn.Module):
             loss_reduction=loss_reduction,
             batch_first=batch_first,
         )
-        grad_sampler_fn = self.GRAD_SAMPLERS[type(module)]
+        if not self.force_functorch and type(module) in self.GRAD_SAMPLERS:
+            grad_sampler_fn = self.GRAD_SAMPLERS[type(module)]
+        else:
+            grad_sampler_fn = ft_compute_per_sample_gradient
+
         grad_samples = grad_sampler_fn(module, activations, backprops)
         for param, gs in grad_samples.items():
             create_or_accumulate_grad_sample(
@@ -397,16 +378,14 @@ class GradSampleModule(nn.Module):
 
         batch_dim = 0 if batch_first or type(module) is RNNLinear else 1
 
-        activations = module.activations.pop()
-
         if not hasattr(module, "max_batch_len"):
             # For packed sequences, max_batch_len is set in the forward of the model (e.g. the LSTM)
             # Otherwise we infer it here
             module.max_batch_len = _get_batch_size(
                 module=module,
-                grad_sample=activations,
                 batch_dim=batch_dim,
             )
+        activations = module.activations.pop()
 
         n = module.max_batch_len
         if loss_reduction == "mean":
@@ -420,9 +399,10 @@ class GradSampleModule(nn.Module):
 
         # No matter where the batch dimension was, .grad_samples will *always* put it in the first dim
         if batch_dim != 0:
-            activations = activations.permute(
-                [batch_dim] + [x for x in range(activations.dim()) if x != batch_dim]
-            )
+            activations = [
+                t.permute([batch_dim] + [x for x in range(t.dim()) if x != batch_dim])
+                for t in activations
+            ]
             backprops = backprops.permute(
                 [batch_dim] + [x for x in range(backprops.dim()) if x != batch_dim]
             )
@@ -444,10 +424,13 @@ class GradSampleModule(nn.Module):
         Returns:
             ``True`` if grad sampler is found, ``False`` otherwise
         """
-        return type(module) in cls.GRAD_SAMPLERS or isinstance(
-            module, (DPRNNBase, DPRNNCellBase)
+        warnings.warn(
+            "GradSampleModule.is_supported is deprecated, as all layers can now be used with functorch.",
+            DeprecationWarning,
         )
 
+        return True
+
     @classmethod
     def validate(
         cls, module: nn.Module, *, strict: bool = False
@@ -479,7 +462,10 @@ class GradSampleModule(nn.Module):
                     f"(See opacus.grad_sample.utils.register_grad_sampler)"
                 )
                 for m_name, m in trainable_modules(module)
-                if not GradSampleModule.is_supported(m)
+                # With functorch, all modules are trainable
+                # We still want to avoid module that have buffers (e.g. BatchNorm)
+                # as the buffers are not private
+                if len(list(m.buffers())) > 0
             ]
         )
         # raise or return errors as needed
@@ -489,29 +475,25 @@ class GradSampleModule(nn.Module):
             return errors
 
 
-def _get_batch_size(
-    *, module: nn.Module, grad_sample: torch.Tensor, batch_dim: int
-) -> int:
+def _get_batch_size(*, module: nn.Module, batch_dim: int) -> int:
     """
     Computes and returns the maximum batch size which is the maximum of the dimension values
-    along 'batch_dim' axis over module.activations + [grad_sample], where module.activations is
+    along 'batch_dim' axis over module.activations, where module.activations is
     a list.
 
-    If module.activations is a not a list, then return grad_sample.shape[batch_dim].
-
     Args:
         module: input module
-        grad_sample: per sample gradient tensor
         batch_dim: batch dimension
 
     Returns:
         Maximum sequence length in a batch
     """
-
     max_batch_len = 0
     for out in module.activations:
-        if out.shape[batch_dim] > max_batch_len:
-            max_batch_len = out.shape[batch_dim]
+        # out is typically a tuple of one element (x)
+        # for embedding bag, it is a tuple of two elements (x, offsets)
+        # where len(offsets) = batch_size
+        if out[-1].shape[batch_dim] > max_batch_len:
+            max_batch_len = out[-1].shape[batch_dim]
 
-    max_batch_len = max(max_batch_len, grad_sample.shape[batch_dim])
     return max_batch_len
