@@ -1,7 +1,20 @@
 #!/usr/bin/env python3
-# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved
+# Copyright (c) Meta Platforms, Inc. and affiliates.
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
 
 import warnings
+from collections import OrderedDict
 
 import torch
 import torch.nn as nn
@@ -76,11 +89,14 @@ class DPMultiheadAttention(nn.Module):
         add_zero_attn=False,
         kdim=None,
         vdim=None,
+        device=None,
+        dtype=None,
     ):
         super(DPMultiheadAttention, self).__init__()
         self.embed_dim = embed_dim
         self.kdim = kdim if kdim is not None else embed_dim
         self.vdim = vdim if vdim is not None else embed_dim
+        self._qkv_same_embed_dim = self.kdim == embed_dim and self.vdim == embed_dim
 
         self.num_heads = num_heads
         self.dropout = dropout
@@ -93,9 +109,7 @@ class DPMultiheadAttention(nn.Module):
         self.klinear = nn.Linear(self.kdim, embed_dim, bias=bias)
         self.vlinear = nn.Linear(self.vdim, embed_dim, bias=bias)
 
-        # torch.nn.MultiHeadAttention out_proj is _LinearWithBias
-        # explicilty setting bias=True for consistent mimicry
-        self.out_proj = nn.Linear(embed_dim, embed_dim, bias=True)
+        self.out_proj = nn.Linear(embed_dim, embed_dim, bias=bias)
 
         self.add_bias_kv = add_bias_kv
         if self.add_bias_kv:
@@ -317,3 +331,89 @@ class DPMultiheadAttention(nn.Module):
             return attn_output, attn_output_weights.sum(dim=1) / self.num_heads
         else:
             return attn_output, None
+
+    def unsqueeze_0_2(self, t):
+        return torch.unsqueeze(torch.unsqueeze(t, 0), 0)
+
+    def state_dict(self, destination=None, prefix="", keep_vars=False):
+        if destination is None:
+            destination = OrderedDict()
+            destination._metadata = OrderedDict()
+
+        local_metadata = dict(version=self._version)
+        if hasattr(destination, "_metadata"):
+            destination._metadata[prefix[:-1]] = local_metadata
+
+        destination_alter = OrderedDict()
+        if len(prefix.split(".")) > 2:
+            alter_key = ".".join(prefix.split(".")[:-2]) + ".emb.weight"
+        else:
+            alter_key = "emb.weight"
+        if alter_key in destination:
+            destination_alter[alter_key] = destination[alter_key]
+
+        self._save_to_state_dict(destination, prefix, keep_vars)
+        for name, module in self._modules.items():
+            if module is not None:
+                module.state_dict(
+                    destination=destination,
+                    prefix=prefix + name + ".",
+                    keep_vars=keep_vars,
+                )
+
+        if self._qkv_same_embed_dim:
+            destination_alter[prefix + "in_proj_weight"] = torch.cat(
+                (
+                    destination[prefix + "qlinear.weight"],
+                    destination[prefix + "klinear.weight"],
+                    destination[prefix + "vlinear.weight"],
+                ),
+                0,
+            )
+        else:
+            destination_alter[prefix + "q_proj_weight"] = destination[
+                prefix + "qlinear.weight"
+            ]
+            destination_alter[prefix + "k_proj_weight"] = destination[
+                prefix + "klinear.weight"
+            ]
+            destination_alter[prefix + "v_proj_weight"] = destination[
+                prefix + "vlinear.weight"
+            ]
+
+        if (
+            (prefix + "qlinear.bias") in destination
+            and (prefix + "klinear.bias") in destination
+            and (prefix + "vlinear.bias") in destination
+        ):
+            destination_alter[prefix + "in_proj_bias"] = torch.cat(
+                (
+                    destination[prefix + "qlinear.bias"],
+                    destination[prefix + "klinear.bias"],
+                    destination[prefix + "vlinear.bias"],
+                ),
+                0,
+            )
+
+        if self.add_bias_kv:
+            destination_alter[prefix + "bias_k"] = self.unsqueeze_0_2(
+                destination[prefix + "seq_bias_k.bias"]
+            )
+            destination_alter[prefix + "bias_v"] = self.unsqueeze_0_2(
+                destination[prefix + "seq_bias_v.bias"]
+            )
+
+        destination_alter[prefix + "out_proj.weight"] = destination[
+            prefix + "out_proj.weight"
+        ]
+        if (prefix + "out_proj.bias") in destination:
+            destination_alter[prefix + "out_proj.bias"] = destination[
+                prefix + "out_proj.bias"
+            ]
+
+        for hook in self._state_dict_hooks.values():
+            hook_result = hook(self, destination_alter, prefix, local_metadata)
+            if hook_result is not None:
+                destination_alter = hook_result
+
+        return destination_alter
