@@ -16,8 +16,7 @@
 import io
 import logging
 import sys
-from collections import OrderedDict
-from typing import Iterable, List, Tuple
+from typing import Dict, Iterable, List, Tuple
 
 import torch
 import torch.nn as nn
@@ -32,7 +31,11 @@ logger = logging.getLogger(__name__)
 logger.setLevel(level=logging.INFO)
 
 
-def parametrized_modules(module: nn.Module) -> Iterable[nn.Module]:
+def has_trainable_params(module: nn.Module) -> bool:
+    return any(p.requires_grad for p in module.parameters(recurse=False))
+
+
+def parametrized_modules(module: nn.Module) -> Iterable[Tuple[str, nn.Module]]:
     """
     Recursively iterates over all submodules, returning those that
     have parameters (as opposed to "wrapper modules" that just organize modules).
@@ -73,7 +76,7 @@ def requires_grad(module: nn.Module, *, recurse: bool = False) -> bool:
     Args:
         module: PyTorch module whose parameters are to be examined.
         recurse: Flag specifying if the gradient requirement check should
-            be applied recursively to sub-modules of the specified module
+            be applied recursively to submodules of the specified module
 
     Returns:
         Flag indicate if any parameters require gradients
@@ -142,22 +145,19 @@ def get_submodule(module: nn.Module, target: str) -> nn.Module:
     return mod
 
 
-def are_state_dict_equal(sd1: OrderedDict, sd2: OrderedDict):
+def are_state_dict_equal(sd1: Dict, sd2: Dict):
     """
     Compares two state dicts, while logging discrepancies
     """
     if len(sd1) != len(sd2):
-        logger.error(f"Length mismatch: {len(sd1)} vs {len(sd2)}")
         return False
 
     for k1, v1 in sd1.items():
         # check that all keys are accounted for.
         if k1 not in sd2:
-            logger.error(f"Key missing: {k1} not in {sd2}")
             return False
         # check that value tensors are equal.
         v2 = sd2[k1]
         if not torch.allclose(v1, v2):
-            logger.error(f"Tensor mismatch: {v1} vs {v2}")
             return False
     return True
